require "rest-client"
require "redis"
require "json"

require "dingtalk/version"
require "dingtalk/config"
require "dingtalk/handler"
require "dingtalk/api"
require "dingtalk/eco_api"
require "dingtalk/client"

module Dingtalk

  module Token
    autoload(:Store,       "dingtalk/token/store")
    autoload(:ObjectStore, "dingtalk/token/object_store")
    autoload(:RedisStore,  "dingtalk/token/redis_store")
  end

  OK_MSG  = "ok".freeze
  OK_CODE = 0.freeze
  CUSTOM_ENDPOINT = "custom_endpoint".freeze
  ECO_ENDPOINT = 'eco'.freeze

  class << self
    attr_accessor :current_endpoint

    def eco?
      @current_endpoint == ECO_ENDPOINT
    end

    def http_get_without_token(url, url_params={}, endpoint="api")
      @current_endpoint = endpoint
      load_json(resource(url).get(params: url_params), url)
    end

    def http_post_without_token(url, _post_body={}, url_params={}, endpoint="api")
      @current_endpoint = endpoint
      load_json(resource(url).post(JSON.dump(_post_body), params: url_params, content_type: :json), url)
    end

    def resource(url)
      RestClient::Resource.new(endpoint_url(url), rest_client_options)
    end

    def load_json(string, url)
      result_hash = JSON.parse(string.force_encoding("UTF-8").gsub(/[\u0011-\u001F]/, ""))
      if eco?
        EcoResultHandler.new(url, result_hash)
      else
        code   = result_hash.delete("errcode")
        en_msg = result_hash.delete("errmsg")
        ResultHandler.new(code, en_msg, result_hash)
      end
    end

    def endpoint_url(url)
      if eco?
        eco_endpoint
      else
        "#{api_endpoint}" + url
      end
    end

    def api_endpoint
      "https://oapi.dingtalk.com"
    end

    def eco_endpoint
      "https://eco.taobao.com/router/rest"
    end

    def calculate_expire(expires_in)
      Time.now.to_i + expires_in.to_i - key_expired.to_i
    end
  end
end
