require "monitor"
require "redis"
require 'digest/md5'
module Dingtalk
  class Client
    include MonitorMixin
    include Api::Department
    include Api::User
    include Api::Microapp
    include Api::Message

    include EcoApi::Role

    attr_accessor :app_id, :app_secret, :expired_at # Time.now + expires_in
    attr_accessor :access_token, :redis_key

    def initialize(app_id, app_secret, options={})
      @app_id = app_id
      @app_secret = app_secret
      @expired_at = Time.now.to_i
      @redis_key = security_redis_key(options[:redis_key] || "dingtalk_#{app_id}")
      super()
    end

    def get_access_token
      synchronize{ token_store.access_token }
    end

    def is_valid?
      token_store.valid?
    end

    def token_store
      Token::Store.init_with(self)
    end

    def http_get(url, url_params={}, endpoint="plain")
      url_params = endpoint == Dingtalk::ECO_ENDPOINT ? url_params.merge(eco_access_token_param) : url_params.merge(access_token_param)
      Dingtalk.http_get_without_token(url, url_params, endpoint)
    end

    def http_post(url, post_body={}, url_params={}, endpoint="plain")
      url_params = endpoint == Dingtalk::ECO_ENDPOINT ? eco_access_token_param.merge(url_params) : access_token_param.merge(url_params)
      Dingtalk.http_post_without_token(url, post_body, url_params, endpoint)
    end

    private

    def eco_access_token_param
      {session: get_access_token, timestamp: Time.now.strftime("%Y-%m-%d %H:%M:%S"), format: 'json', v: '2.0'}
    end

    def access_token_param
      {access_token: get_access_token}
    end

    def security_redis_key(key)
      Digest::MD5.hexdigest(key.to_s).upcase
    end
  end
end