module Dingtalk
  class EcoResultHandler
    attr_accessor :code, :cn_msg, :en_msg, :result, :response_name

    def initialize(m, response={})
      @response_name = "#{m.gsub(/\./, '_')}_response"
      @response = package_result(response)
      @result = @response[@response_name]
    end

    def is_ok?
      @response[:error_response].nil?
    end
    alias_method :ok?, :is_ok?

    def code
      @response[:error_response][:code]
    end

    def en_msg
      @response[:error_response][:msg]
    end

    def cn_msg
      @response[:error_response][:sub_msg]
    end

    def sub_code
      @response[:error_response][:sub_code]
    end

    def full_message
      if is_ok?
        "SUCCESS, Please use #result to get result."
      else
        "#{code}: #{en_msg}(#{sub_code}:#{cn_msg})."
      end
    end
    alias_method :full_messages, :full_message

    def full_error_message
      full_message if !is_ok?
    end
    alias_method :full_error_messages, :full_error_message

    private
    def package_result(response)
      return response if !response.is_a?(Hash)
      if defined?(Rails)
        ActiveSupport::HashWithIndifferentAccess.new(response)
      else
        response
      end
    end
  end
end